"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const google_auth_library_1 = require("google-auth-library");
const googleapis_1 = require("googleapis");
const util = require("util");
const cloudDebugger = googleapis_1.google.clouddebugger('v2').debugger;
class Wrapper {
    constructor() {
        this.googleAuth = new google_auth_library_1.GoogleAuth();
        this.waitToken = '';
    }
    isDebuggee(schemaDebuggee) {
        const debuggee = schemaDebuggee;
        return typeof debuggee.id === 'string' && debuggee.labels &&
            typeof debuggee.labels.projectid === 'string' &&
            typeof debuggee.labels.version === 'string';
    }
    isDebuggeeList(debuggeeList) {
        for (const debuggee of debuggeeList) {
            if (!this.isDebuggee(debuggee)) {
                return false;
            }
        }
        return true;
    }
    isBreakpoint(breakpointRequest) {
        const breakpoint = breakpointRequest;
        return typeof breakpoint.id === 'string' && breakpoint.location &&
            typeof breakpoint.location.path === 'string' &&
            typeof breakpoint.location.line === 'number';
    }
    isPendingBreakpointList(breakpointList) {
        for (const breakpoint of breakpointList) {
            // A pending breakpoint list should not have any captured snapshots.
            if (!this.isBreakpoint(breakpoint) || breakpoint.isFinalState) {
                return false;
            }
        }
        return true;
    }
    authorize(keyFilename) {
        return __awaiter(this, void 0, void 0, function* () {
            const credential = yield this.googleAuth.getClient({
                keyFilename,
                scopes: ['https://www.googleapis.com/auth/cloud-platform'],
            });
            if (!credential.projectId) {
                throw new Error('The given keyFile must contain the project ID.');
            }
            this.projectId = credential.projectId;
            this.auth = credential;
        });
    }
    getProjectId() {
        if (!this.projectId) {
            throw new Error('You must select a project before continuing.');
        }
        return this.projectId;
    }
    debuggeesList() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.auth) {
                throw new Error('You must select a project before continuing.');
            }
            const request = {
                project: this.projectId,
                auth: this.auth,
            };
            const response = yield cloudDebugger.debuggees.list(request);
            if (response.data.debuggees === undefined) {
                return [];
            }
            else if (this.isDebuggeeList(response.data.debuggees)) {
                return response.data.debuggees;
            }
            else {
                throw new Error('The debuggees.list response from Stackdriver Debug ' +
                    'contains an element that is not a debuggee: ' +
                    util.inspect(response.data, { depth: null }));
            }
        });
    }
    debuggeesBreakpointsDelete(breakpointId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.auth) {
                throw new Error('You must select a project before continuing.');
            }
            if (!this.debuggeeId) {
                throw new Error('You must select a debuggee before continuing.');
            }
            const request = {
                debuggeeId: this.debuggeeId,
                breakpointId,
                auth: this.auth,
            };
            const response = yield cloudDebugger.debuggees.breakpoints.delete(request);
        });
    }
    debuggeesBreakpointsGet(breakpointId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.auth) {
                throw new Error('You must select a project before continuing.');
            }
            if (!this.debuggeeId) {
                throw new Error('You must select a debuggee before continuing.');
            }
            const request = {
                debuggeeId: this.debuggeeId,
                breakpointId,
                auth: this.auth,
            };
            const response = yield cloudDebugger.debuggees.breakpoints.get(request);
            if (!response.data.breakpoint ||
                !this.isBreakpoint(response.data.breakpoint)) {
                throw new Error('The debuggees.breakpoints.get response from Stackdriver Debug is ' +
                    `missing a property: ${util.inspect(response.data, { depth: null })}`);
            }
            return response.data.breakpoint;
        });
    }
    debuggeesBreakpointsList(wait) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.auth) {
                throw new Error('You must select a project before continuing.');
            }
            if (!this.debuggeeId) {
                throw new Error('You must select a debuggee before continuing.');
            }
            const request = {
                debuggeeId: this.debuggeeId,
                waitToken: wait ? this.waitToken : '',
                auth: this.auth,
            };
            const response = yield cloudDebugger.debuggees.breakpoints.list(request);
            if (!response.data.nextWaitToken) {
                throw new Error('The debuggees.breakpoints.list response from Stackdriver Debug ' +
                    'should have the nextWaitToken property, but it returned this: ' +
                    util.inspect(response.data, { depth: null }));
            }
            this.waitToken = response.data.nextWaitToken;
            if (response.data.breakpoints === undefined) {
                return [];
            }
            else if (this.isPendingBreakpointList(response.data.breakpoints)) {
                return response.data.breakpoints;
            }
            else {
                throw new Error('The debuggees.breakpoints.list response from Stackdriver Debug ' +
                    'contains an element that is not a pending breakpoint: ' +
                    util.inspect(response.data.breakpoints, { depth: null }));
            }
        });
    }
    debuggeesBreakpointsSet(breakpoint) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.auth) {
                throw new Error('You must select a project before continuing.');
            }
            if (!this.debuggeeId) {
                throw new Error('You must select a debuggee before continuing.');
            }
            const request = {
                debuggeeId: this.debuggeeId,
                requestBody: breakpoint,
                auth: this.auth,
            };
            const response = yield cloudDebugger.debuggees.breakpoints.set(request);
            if (!response.data.breakpoint ||
                !this.isBreakpoint(response.data.breakpoint)) {
                throw new Error('The debuggees.breakpoints.set response from Stackdriver Debug ' +
                    `missing a property: ${util.inspect(response.data, { depth: null })}`);
            }
            return response.data.breakpoint;
        });
    }
}
exports.Wrapper = Wrapper;
//# sourceMappingURL=wrapper.js.map